// Enemy Class
// MY CHYE - Abertay Dundee Uni 2009

#include "Enemy.h"
#include <stdio.h>


// MACROS

#define INSIDE_ANGLE 30 // inside cone angle for zombie voice
#define OUTSIDE_ANGLE 60 // outside cone angle for zombie voice
#define CONE_OUTSIDE_VOLUME -2000

Enemy::Enemy(DSoundDevice* device)
{
	theDevice = device;	
	theDevice->CreateSoundBuffer(&groan1, "zombiegroan1.wav", DSBCAPS_CTRL3D );
	groan3d1 = groan1->Get3dBuffer();
	theDevice->CreateSoundBuffer(&groan2, "zombiegroan2.wav", DSBCAPS_CTRL3D);
	groan3d2 = groan2->Get3dBuffer();
	theDevice->CreateSoundBuffer(&attack, "zombieattack.wav",  DSBCAPS_CTRL3D);	
	attack3d = attack->Get3dBuffer();
	theDevice->CreateSoundBuffer(&scream, "monsterscream.wav", DSBCAPS_CTRL3D);	
	scream3d = scream->Get3dBuffer();
	theDevice->CreateSoundBuffer(&suspect, "zombiesuspicious.wav", DSBCAPS_CTRL3D);	
	suspect3d = suspect->Get3dBuffer();
	orient.y = 0; 
	orient.x = 0;
	orient.z = 1; // face north initially
	position.x = 0;
	position.y = 0;
	position.z = 0;
	alive = true;
	groan = false;
	health = 100;	
	groaned = false;
	attacking = false;	
}

Enemy::~Enemy()
{
	if (groan1)   free (groan1);
	if (groan2)   free (groan2);
	if (attack)   free (attack);
	if (suspect) free (suspect);
	if (scream) free (scream);
	if (groan3d1) groan3d1->Release();
	if (groan3d2) groan3d2->Release();
	if (attack3d) attack3d->Release();
	if (suspect3d) attack3d->Release();
	if (scream3d) scream3d->Release();
	groan1 = NULL;
	groan2 = NULL;
	attack = NULL;
	suspect = NULL;
	suspect3d = NULL;
	groan3d1 = NULL;
	groan3d2 = NULL;
	attack3d = NULL;
	scream = NULL;
	scream3d = NULL;
}

void Enemy::Init(int MaxX, int MinX, int MaxZ, int MinZ, float rad, int attacks) // Position and orientation on ZX plane ; rad is their attack radius
{
	attackspeed = attacks;
	// Gets a random value within the range
	//little tweak so it wont spawn to close to the walls
	static int tempX = ((MaxX-(rand()%3+2)) - (MinX+(rand()%3+2)));
	static int tempZ = ((MaxZ-(rand()%3+2)) - (MinZ+(rand()%3+2)));
	static int finalX = tempX; //little tweak so it wont spawn to close to the walls
	static int finalZ = tempZ;	
	/*position.x = 0;
	position.y = 0;*/
	position.x = rand()%finalX - MaxX;
	position.y = 0;
	position.z = rand()%finalZ - MaxZ;

	static int tempXX = (rand()%3 - 1); // range from -1 to 1
	static int tempYY = (rand()%3 - 1);

	static int rand1 = 0;
	rand1 = rand()%4;
	if (rand1 == 0)
	{
		orient.x = 0;
		orient.z = 1;
	}
	else if (rand1 == 1)
	{
		orient.x = 0;
		orient.z = -1;
	}
	else if (rand1 == 2)
	{
		orient.x = 1;
		orient.z = 0;
	}
	else if (rand1 == 3)
	{
		orient.x = -1;
		orient.z = 0;
	}	

	if (tempXX == tempYY) tempXX = (rand()%3-1);
	orient.x = tempXX;
	orient.z = tempYY;
	
	// Chooses 1 random zombie sfx to play
	if(rand()%2 == 0)
	{
		groaned = true;
		groan3d1->SetPosition(position.x, 0, position.z , DS3D_IMMEDIATE);		
		// With this theory , hopefully the player will know where the zombie is "facing"
		//	groan3d1->SetConeAngles(INSIDE_ANGLE, OUTSIDE_ANGLE, DS3D_IMMEDIATE);
		//	groan3d1->SetConeOrientation(orientx, 0, orientz, DS3D_IMMEDIATE);
		//	groan3d1->SetConeOutsideVolume(CONE_OUTSIDE_VOLUME, DS3D_IMMEDIATE);
	}
	else 
	{
		//	groan3d2->SetConeOrientation(orientx, 0, orientz, DS3D_IMMEDIATE);
		//	groan3d2->SetConeOutsideVolume(CONE_OUTSIDE_VOLUME, DS3D_IMMEDIATE);
		groan3d2->SetPosition(position.x, 0, position.z , DS3D_IMMEDIATE);
	}
	attack3d->SetPosition(position.x, 0, position.z, DS3D_IMMEDIATE);	
	suspect3d->SetPosition(position.x, 0, position.z, DS3D_IMMEDIATE);
	reach = rad; // radius around the Enemy which it could react
	alive = true;	
	attackspeed = attacks;
}

void Enemy::Suspicious()
{
	if(alive)
	{
		if(groaned)
		{
			if(groan1->IsPlaying())groan1->Stop();
		}
		if(groan2->IsPlaying())groan2->Stop();
		if(attack->IsPlaying())attack->Stop();
		static int count = 50;
		static bool played = false;
		if(!played)
		{
			if(!suspect->IsPlaying())suspect->Play();
			played = true;
		}
		if(count == 0)
		{
			played = false;
			count = 50;
		}

		count--;
	}
}



int Enemy::GetX()
{
	return position.x;
}

int Enemy::GetZ()
{
	return position.z;
}

float Enemy::GetRad()
{
	return reach;
}

void Enemy::Normal()
{
	if(alive)
	{
		if(attack->IsPlaying())attack->Stop();
		if(suspect->IsPlaying())suspect->Stop();
		PlaytheSound();
	}
}


void Enemy::Attack()
{
	if(alive)
	{
		if(groaned) groan1->Stop();
		else groan2->Stop();
		if(suspect) suspect->Stop();

		static int count = attackspeed; // locked @ 60 fps 
		static bool played = false;
		if(!attack->IsPlaying() && !played)
		{
			D3DVECTOR temp;
			//attack3d->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
			attack->Play();
			count--;	
			//printf("ATTACK()!\n\n\n");
			attacking = true;
			played = true;
		}
		if(count == 0)
		{
			played = false;
			count = attackspeed;
			attacking = false;
		}
		count--;
	}		
}

void Enemy::Die()
{
	if(alive)
	{
		if(groaned) groan1->Stop();
		else groan2->Stop();
		if(suspect) suspect->Stop();
		
		if(!scream->IsPlaying())scream->Play();
		alive = false;
	}
}
void Enemy::PlaytheSound()
{
	if(alive)
	{
		static int count = 280; // locked @ 60 fps so 180 is around 3 secs delay
		static bool played1 = false;
		if(groaned)
		{
			if(!groan1->IsPlaying() && !played1)
			{
				groan1->Play();
				played1 = true;
			}
			if(count == 0)
			{
				played1 = false;
				count =280;
			}
			count--;
		}
		
		else if(!groaned)
		{

			if(!groan2->IsPlaying() && !groan)
			{
				groan2->Play();
				groan = true;
			}
			if(count == 0)
			{
				groan = false;
				count = 280;
			}
			count--;
		}
	}
}

CVector2 Enemy::Return2Dpos()
{
	CVector2 pos(position.x, position.z);
	return pos;
}

CVector2 Enemy::Return2DOrient()
{
	CVector2 orient(orient.x, orient.z);
	return orient;
}

int Enemy::GetAttackSpeed()
{
	return attackspeed;
}

void Enemy::StopAllSound()
{
	if(alive)
	{
	if(groan1->IsPlaying())groan1->Stop();
	if(groan2->IsPlaying())groan2->Stop();
	if(attack->IsPlaying())attack->Stop();
	if(scream->IsPlaying())scream->Stop();
	}

}

bool Enemy::GetStatus()
{
	return alive;
}

int Enemy::GetOrX()
{
	return orient.x;
}

int Enemy::GetOrZ()
{
	return orient.z;
}
void Enemy::KillAll()
{
	alive = false;
}