// Player Class
// MY CHYE - Abertay Dundee Uni 2009

#include "Player.h"
#include <stdio.h> 
#define REACH 2

Player::Player(DSoundDevice* device, int Lives)
{
	reach = REACH; // radius around player used for collision detection etc
	life = Lives;
	MaxLife = Lives;
	player = device->Get3dListener();
	player->SetPosition(0, 0, 0, DS3D_IMMEDIATE);
	player->SetOrientation(0, 0, 1, 0, 1, 0, DS3D_IMMEDIATE);
	player->SetRolloffFactor(3, DS3D_IMMEDIATE);	
	dsDevice = device;
	//bump into wall sounds
	dsDevice->CreateSoundBuffer(&walkSound, "footstep.wav", DSBCAPS_CTRL3D);
	dsDevice->CreateSoundBuffer(&bumpWallNorth, "bumpnorth.wav", DSBCAPS_CTRL3D);
	dsDevice->CreateSoundBuffer(&bumpWallSouth, "bumpsouth.wav", DSBCAPS_CTRL3D);
	dsDevice->CreateSoundBuffer(&bumpWallEast, "bumpEast.wav", DSBCAPS_CTRL3D);
	dsDevice->CreateSoundBuffer(&bumpWallWest, "bumpWest.wav", DSBCAPS_CTRL3D);

	playerSound = walkSound->Get3dBuffer();
	playerSound->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
	playerSound = bumpWallNorth->Get3dBuffer();
	playerSound->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
	playerSound = bumpWallSouth->Get3dBuffer();
	playerSound->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
	playerSound = bumpWallEast->Get3dBuffer();
	playerSound->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
	playerSound = bumpWallWest->Get3dBuffer();
	playerSound->SetMode(DS3DMODE_HEADRELATIVE, DS3D_IMMEDIATE);
	
	// Init our axis for player
	right.set(1.0f, 0.0f, 0.0f); // +X axis
	up.set(0.0f, 1.0f, 0.0f); // +Y axis
	forward.set(0.0f, 0.0f, 1.0f); // +Z axis	
	walk1= false;
	bump = false;
	moveFor = true;
	moveBack = true;
	moveLeft = true;
	wall = false;
}

Player::~Player()
{
	if (player) player->Release();
	if (walkSound) free (walkSound);
	if (bumpWallNorth) free (bumpWallNorth);
	if (bumpWallSouth) free (bumpWallSouth);
	if (bumpWallEast) free (bumpWallEast);
	if (bumpWallWest) free (bumpWallWest);
	player = NULL;
	walkSound = NULL;
	bumpWallNorth = NULL;
	bumpWallSouth = NULL;
	bumpWallEast = NULL;
} 

void Player::Reset()
{
	life = MaxLife;
	player->SetPosition(0, 0, 0, DS3D_IMMEDIATE);
	player->SetOrientation(0, 0, 1, 0, 1, 0, DS3D_IMMEDIATE);

	right.set(1.0f, 0.0f, 0.0f); // +X axis
	up.set(0.0f, 1.0f, 0.0f); // +Y axis
	forward.set(0.0f, 0.0f, 1.0f); // +Z axis	

	// DEBUG
	PrintDebug();
}

void Player::playBumpSound(int i)
{

	if(i == 1)
	{
		if(!bumpWallNorth->IsPlaying() && !bump)
		{
			bumpWallNorth->Play();
			bump = true;

		}
	}
	if(i == 2)
	{
		if(!bumpWallEast->IsPlaying() && !bump)
		{
			bumpWallEast->Play();
			bump = true;

		}
	}
	if(i == 3)
	{
		if(!bumpWallSouth->IsPlaying() && !bump)
		{
			bumpWallSouth->Play();
			bump = true;

		}
	}
	if(i == 4)
	{
		if(!bumpWallWest->IsPlaying() && !bump)
		{
			bumpWallWest->Play();
			bump = true;

		}
	}

	if(GetTickCount() % 10 == 0)
	{
		bump = false;		
	}
}

void Player::stopWalkSound()
{
	if(walkSound->IsPlaying())walkSound->Stop();
}

void Player::playWalkSound()
{
	static int count = 3;
	static bool played = false;
	if(!played)
	{
		if(!walkSound->IsPlaying())walkSound->Play();
		played = true;
	
	}
	if(count == 0)
	{
		played = false;
		count = 3;
	}
	count--	;
}

bool Player::TestBoundary() // makes sure player is inside the predetermined boundary
{
	D3DVECTOR temp;
	player->GetPosition(&temp);
	if(temp.x < MIN_X)
	{

		playBumpSound(WEST);		
		//MouseInput();
		//player->SetPosition(temp.x + 0.001f, temp.y, temp.z, DS3D_IMMEDIATE);	
		
		return false;
	}

	else if(temp.x > MAX_X)
	{
		playBumpSound(EAST);
		//MouseInput();
		//player->SetPosition(temp.x - 0.001f, temp.y, temp.z, DS3D_IMMEDIATE);
		
		return false;
	}
	else if(temp.z < MIN_Z)
	{
		playBumpSound(SOUTH);
		//MouseInput();
		//player->SetPosition(temp.x, temp.y, temp.z + 0.001f, DS3D_IMMEDIATE);		
		
		return false;
	}
	else if(temp.z > MAX_Z)
	{

		playBumpSound(NORTH);
		//MouseInput();
		//player->SetPosition(temp.x, temp.y, temp.z - 0.001f, DS3D_IMMEDIATE);	
		return false;
	}
	return true;	
}


void Player::PrintDebug() //prints debug information
{
	//for debug
	D3DVECTOR tempPos, tempFront, tempTop;
	player->GetPosition(&tempPos);
	player->GetOrientation(&tempFront, &tempTop);
	printf("\nPlayer position x:%f  y:%f  z:%f \n", tempPos.x, tempPos.y, tempPos.z);
	printf("Player orientation x:%f y:%f z:%f\n", tempFront.x, tempFront.y, tempFront.z);	
	printf("Top X:%f, Y:%f, Z:%f,\n\n", tempTop.x, tempTop.y, tempTop.z);
}

void Player::rotateY(float angle)
{
	D3DVECTOR oldOrient, newOrient, newTop;
	float xxx, zzz;
	float sinAng, cosAng;
	sinAng = sinf(angle);
	cosAng = cosf(angle);
	xxx = forward.x;
	zzz = forward.z;	
	// Rotate forward vector
	forward.x = xxx * cosAng + zzz * sinAng;
	forward.z = xxx * -sinAng + zzz * cosAng;
	newOrient.x = forward.x;
	newOrient.z = forward.z;
	xxx = up.x;
	zzz = up.z;	
	// Rotate up vector
	up.x = xxx * cosAng + zzz * sinAng;
	up.z = xxx * -sinAng + zzz * cosAng;
	newTop.x = up.x;
	newTop.z = up.z;
	xxx = right.x;
	zzz = right.z;	
	// Rotate right vector
	right.x = xxx * cosAng + zzz * sinAng;
	right.z = xxx * -sinAng + zzz * cosAng;
	player->GetOrientation(&oldOrient, &newTop);	
	player->SetOrientation(newOrient.x, oldOrient.y, newOrient.z, 0, 1, 0, DS3D_IMMEDIATE);
	//printf("New Orient x: %f\n\nNew Orient z: %f\n\n", newOrient.x, newOrient.z);
}

// Moves the camera in the specified direction
void Player::move(MoveDir dir, float amt)
{	
	CVector newPos;
	D3DVECTOR pos,Orient, OrientTop;
	player->GetOrientation(&Orient, &OrientTop);
	forward.x = Orient.x;
	forward.y = Orient.y;
	forward.z = Orient.z;
	player->GetPosition(&pos);
	switch(dir)
	{
	case eForward: // Move along the forward vector
		if(moveFor)
		{
			moveBack = true;			
			moveRight = true;
			moveLeft = true;
			wall = false;
			newPos+= (forward * amt);
			player->SetPosition(pos.x + newPos.x, newPos.y, pos.z + newPos.z, DS3D_IMMEDIATE);
		}

		if(!TestBoundary())
		{					
			moveFor = false;
			moveBack = true;
			moveRight = true;
			moveLeft = true;	
			wall = true;
		}

		// DEBUG
		PrintDebug();
		break;		

	case eBack: // Move along the negative forward vector
		if(moveBack)
		{
			moveFor = true;			
			moveRight = true;
			moveLeft = true;
			wall = false;
			newPos-= (forward * amt);
			player->SetPosition(pos.x + newPos.x, newPos.y, pos.z + newPos.z, DS3D_IMMEDIATE);
		}

		if(!TestBoundary())
		{					
			moveBack = false;
			moveFor = true;
			moveRight = true;
			moveLeft = true;	
			wall = true;
		}

		// DEBUG
		PrintDebug();
		break;

		
	case eLeft: // Move along the negative right vector
		if(moveLeft)
		{		
			moveFor = true;			
			moveRight = true;
			moveBack = true;
			wall = false;
			newPos -= (right * amt);
			player->SetPosition(pos.x + newPos.x, newPos.y, pos.z + newPos.z, DS3D_IMMEDIATE);
		}
		if(!TestBoundary())
		{					
			moveBack = true;
			moveFor = true;
			moveRight = true;
			moveLeft = false;
			wall = true;

		}
		// DEBUG
		PrintDebug();
		break;

	case eRight: // Move along the right vector
		if(moveRight)
		{	
			moveFor = true;			
			moveBack = true;
			moveLeft = true;
			wall = false;
			newPos += (right * amt);
			player->SetPosition(pos.x + newPos.x, newPos.y, pos.z + newPos.z, DS3D_IMMEDIATE);
		}
		if(!TestBoundary())
		{					
			moveBack = true;
			moveFor = true;
			moveRight = false;
			moveLeft = true;
			wall = true;

		}
		// DEBUG
		PrintDebug();
		break;
	}

}

float Player::GetX()
{
	D3DVECTOR temp;
	player->GetPosition(&temp);
	return temp.x;
}

float Player::GetZ()
{
	D3DVECTOR temp;
	player->GetPosition(&temp);
	return temp.z;
}


// Rotates around an arbritrary axis
CVector Player::rotateAroundAxis(const CVector &vec, const CVector &axis, float angle)
{
	float aX = axis.x; 
	float aY = axis.y; 
	float aZ = axis.z;
	float sinAng = sinf(angle);
	float cosAng = cosf(angle);

	// Calculate X component
	float xxx = (aX * aX * (1.0f - cosAng) + cosAng) * vec.x +
		(aX * aY * (1.0f - cosAng) + aZ * sinAng) * vec.y +
		(aX * aZ * (1.0f - cosAng) - aY * sinAng) * vec.z;

	// Calculate Y component	   
	float yyy = (aX * aY * (1.0f - cosAng) - aZ * sinAng) * vec.x +
		(aY * aY * (1.0f - cosAng) + cosAng) * vec.y +
		(aY * aZ * (1.0f - cosAng) + aX * sinAng) * vec.z;

	// Calculate Z component
	float zzz = (aX * aZ * (1.0f - cosAng) + aY * sinAng) * vec.x +
		(aY * aZ * (1.0f - cosAng) - aX * sinAng) * vec.y +
		(aZ * aZ * (1.0f - cosAng) + cosAng) * vec.z;

	return CVector(xxx, yyy, zzz);
}

// Handles mouse input to rotate the listener
void Player::MouseInput()
{
		// Get the middle of the screen
	int midScrX = GetSystemMetrics(SM_CXSCREEN) >> 1;
	int midScrY = GetSystemMetrics(SM_CYSCREEN) >> 1;
	if(!wall) // stop changing orientation if stuck at wall or this will cause confusion
	{
	float amt;
	POINT pt = {0};	
	GetCursorPos(&pt); // Get the current mouse position
		
	// Rotate left/right
	amt = float(pt.x - midScrX) * kMoveAmt;

	// Rotate left/right
	amt = float(pt.x - midScrX) * kMoveAmt;
	rotateY(DEG2RAD(amt));
	SetCursorPos(midScrX, midScrY); 
	}
	SetCursorPos(midScrX, midScrY); 
}	

float Player::GetRad()
{
	return reach;
}

bool Player::BoundingCircle(float x, float z, float r)
{
	float xsq, zsq, rsq, temp;	
	D3DVECTOR postemp;
	player->GetPosition(&postemp);	
	xsq = pow((x-postemp.x), 2);
	zsq = pow((z-postemp.z), 2);
	rsq = pow((r + REACH), 2);
	temp = (xsq + zsq);
	return (temp < rsq); // collision	
}


bool Player::Attacked(float x, float z, float r, CVector2 Tarpos, CVector2 TarOrient, float tarFOV )
{
	D3DVECTOR temp, tempOr, tempTop;
	player->GetPosition(&temp);	
	//player->GetOrientation(&tempOr, &tempTop);
	CVector2 player(temp.x, temp.z);
	
	if(BoundingCircle(x, z, r) && check_fov(Tarpos, TarOrient, tarFOV, player) ) //within range (player can attack but zombie will depend if it is facing player
	{		
		return true; // check if within zombie fov target = player in this case
	}
	else return false;
}

float Player::unsigned_angle(CVector2 a, CVector2 b)
{
    return RAD2DEG(abs(atan2(perp_dot(a,b), vec_dot(a,b))));
}

bool Player::check_fov(CVector2 Zombiepos, CVector2 zombieOrient, float zombiefov, CVector2 playerPos)
{
	CVector2 TEST(vec_sub(playerPos, Zombiepos));
	float test1 = unsigned_angle(zombieOrient, vec_sub(playerPos , Zombiepos));
    return unsigned_angle(zombieOrient, vec_sub(playerPos , Zombiepos)) < zombiefov / 2;	
}

CVector2 Player::Get2Dpos()
{
	D3DVECTOR temp;
	player->GetPosition(&temp);
	return CVector2(temp.x, temp.y);
}

void Player::LifeCounter(int attackspeed) // minus a life everytime player is hit - cheap trick
{
		static int count = attackspeed;
		static bool played = false;
		if(!played)
		{
			life--;
			printf("\nminus 1 life!\n");
			printf("\nLife atm : %d\n", life);
			played = true;
		}		
		if(count == 0)
		{
			played = false;
			count = attackspeed;
		}	
		count--;
}
		
bool Player::CheckAlive()
{
	if (life <= 0) return false;
	else return true;
}

void Player::SetLife(int iLife)
{
	life = iLife;
}


int Player::GetLife()
{ 
	return life;
}

float Player::GetMaxX()
{
	return MAX_X;
}

float Player::GetMinX()
{
	return MIN_X;
}

float Player::GetMaxZ()
{
	return MAX_Z;
}

float Player::GetMinZ()
{
	return MIN_Z;
}

void Player::FaceNorth()
{
	player->SetOrientation(0, 0, 1, 0, 1, 0, DS3D_IMMEDIATE); // faces player back north
}
